import os

from babel.messages import Catalog
from babel.messages.extract import check_and_call_extract_file, DEFAULT_KEYWORDS
from babel.messages.pofile import write_po
from django.core.management import BaseCommand

from i18nkit.jinja2_extract import jinja2_extract
from i18nkit.utils import add_paths_options, DirectoryFilter, get_paths

METHOD_MAP = [
    ('**.js', 'javascript'),
    ('**.py', 'python'),
    ('**/templates/**.html', 'django'),
    ('**/templates/**.jinja', jinja2_extract),
]

OPTIONS_MAP = {
    '**/templates/**.jinja': {'newstyle_gettext': 'true'},
}

KEYWORDS = dict(DEFAULT_KEYWORDS.copy())
for kw in set(kw for kw in KEYWORDS if kw.isalpha()):
    KEYWORDS['%s_lazy' % kw] = KEYWORDS[kw]


def extract_from_dir(dirname, method_map, options_map, keywords, dirname_filter=None, status_callback=None):
    absname = os.path.abspath(dirname)
    for root, dirnames, filenames in os.walk(absname):
        if dirname_filter:
            dirnames[:] = [dirname for dirname in dirnames if dirname_filter(os.path.join(root, dirname))]
        dirnames.sort()
        filenames.sort()
        for filename in filenames:
            filepath = os.path.join(root, filename).replace(os.sep, '/')

            for message_tuple in check_and_call_extract_file(
                filepath,
                method_map=method_map,
                options_map=options_map,
                callback=status_callback,
                keywords=keywords,
                dirpath=absname,
                comment_tags=(),
                strip_comment_tags=False,
            ):
                yield message_tuple


class Command(BaseCommand):
    def add_arguments(self, parser):
        """
        :type parser: argparse.ArgumentParser
        """
        parser.add_argument('-o', '--output')
        add_paths_options(parser)
        DirectoryFilter.add_options(parser)
        parser.add_argument('-H', '--omit-header', action='store_true')
        parser.add_argument('-z', '--zero-lineno', action='store_true')

    def handle(self, **options):
        self.config = options
        self.dirname_filter = DirectoryFilter(
            ignore_dirs=options['ignore_dirs'],
            default_ignore_dirs=options['default_ignore_dirs'],
        )
        paths = get_paths(
            apps=self.config['apps'],
            dirs=self.config['dirs'],
            dir_children=self.config['dir_children'],
            all_apps=self.config['all_apps'],
            dirname_filter=self.dirname_filter,
        )
        catalog = self.build_catalog(paths)
        self.write_catalog(catalog)

    def write_catalog(self, catalog):
        output_path = self.config['output']
        if not output_path:
            self.stdout.write('would write POT with %d entries, but no `-o` given' % len(catalog))
            return
        self.stdout.write('writing POT with %d entries to %s' % (len(catalog), output_path))
        with open(output_path, 'wb') as outfile:
            write_po(
                outfile,
                catalog,
                width=0,
                omit_header=self.config['omit_header'],
                sort_output=True
            )

    def build_catalog(self, paths):
        catalog = Catalog(charset='UTF-8')
        for path in sorted(paths):
            if self.config['verbosity'] > 1:
                self.stdout.write('processing %s' % path)
            extracted = extract_from_dir(
                path,
                method_map=METHOD_MAP,
                options_map=OPTIONS_MAP,
                keywords=KEYWORDS,
                status_callback=self.status_callback,
                dirname_filter=self.dirname_filter,
            )

            for filename, lineno, message, comments, context in extracted:
                filepath = os.path.normpath(os.path.join(path, filename))
                if self.config['zero_lineno']:
                    lineno = 0

                catalog.add(message, None, [(filepath, lineno)], auto_comments=comments, context=context)
        return catalog

    def status_callback(self, filename, method, options):
        if self.config['verbosity'] > 2:
            self.stdout.write('... %s: %s (%s)' % (filename, method, options))



import os

from babel.messages.pofile import read_po, write_po
from django.conf import settings
from django.core.management import BaseCommand

from i18nkit.excel import read_catalog_workbook
from i18nkit.utils import merge_catalogs


class Command(BaseCommand):
    def add_arguments(self, parser):
        """
        :type parser: argparse.ArgumentParser
        """
        parser.add_argument('-i', '--input')
        parser.add_argument('-o', '--output-template')
        parser.add_argument('-s', '--source-language', default=settings.LANGUAGES[0][0])

    def handle(self, input, output_template, source_language, **options):
        if input.endswith('xlsx'):
            with open(input, 'rb') as infp:
                catalogs = read_catalog_workbook(infp)
        else:  # pragma: no cover
            raise ValueError('unknown input type: %s' % input)

        for locale, update_catalog in sorted(catalogs.items()):
            if locale == source_language:
                continue
            self.write_updated_catalog(output_template, update_catalog)

    def write_updated_catalog(self, output_template, update_catalog):
        locale = str(update_catalog.locale)
        if output_template:
            output_filename = os.path.abspath(output_template.replace('{locale}', locale))
            output_dir = os.path.dirname(output_filename)
            if not os.path.isdir(output_dir):
                os.makedirs(output_dir)
        else:
            output_filename = None

        # Start with the template catalog,

        if output_filename and os.path.isfile(output_filename):
            with open(output_filename, 'rb') as infp:
                existing_catalog = read_po(infp, locale)
        else:
            existing_catalog = None

        # merge them in,
        catalog = merge_catalogs([existing_catalog, update_catalog])

        # And prune translationless entries
        for msg in list(catalog):
            if not msg.string:
                catalog.delete(msg.id)

        if output_filename:
            # and write things out
            with open(output_filename, 'wb') as outfp:
                write_po(outfp, catalog, width=0, sort_output=True, omit_header=True)
            self.stdout.write('Wrote %d entries in %s to %s' % (len(catalog), catalog.locale, output_filename))
        else:
            self.stdout.write('Would have written %d entries in %s' % (len(catalog), catalog.locale))



import os

from babel.messages import Catalog
from babel.messages.mofile import write_mo
from babel.messages.pofile import read_po
from django.core.management import BaseCommand
from django.utils.six import BytesIO

from i18nkit.utils import add_paths_options, DirectoryFilter, get_paths


class Command(BaseCommand):
    def add_arguments(self, parser):
        """
        :type parser: argparse.ArgumentParser
        """
        add_paths_options(parser)
        DirectoryFilter.add_options(parser)

    def handle(self, **options):
        self.config = options
        self.dirname_filter = DirectoryFilter(
            ignore_dirs=options['ignore_dirs'],
            default_ignore_dirs=options['default_ignore_dirs'],
        )
        paths = get_paths(
            apps=self.config['apps'],
            dirs=self.config['dirs'],
            dir_children=self.config['dir_children'],
            all_apps=self.config['all_apps'],
            dirname_filter=self.dirname_filter,
        )
        for root in paths:
            self.process_dir(root)

    def process_dir(self, root):
        for dirpath, dirnames, filenames in os.walk(root):
            dirnames[:] = [dirname for dirname in dirnames if self.dirname_filter(os.path.join(root, dirname))]
            for file in filenames:
                if not file.endswith('.po'):
                    continue
                self.process_file(os.path.join(dirpath, file))

    def process_file(self, file):
        outfile = '%s.mo' % os.path.splitext(file)[0]
        try:
            with open(file, 'rb') as infp:
                catalog = read_po(infp)
            out_bio = BytesIO()
            write_mo(out_bio, catalog)
            with open(outfile, 'wb') as outfp:
                outfp.write(out_bio.getvalue())
        except Exception as exc:
            self.stderr.write('could not process %s: %s' % (file, exc))
        else:
            self.stdout.write('compiled %s -> %s' % (file, outfile))



from warnings import warn

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from jinja2.ext import _CommentFinder, extract_from_ast, babel_extract


def django_jinja2_extract(fileobj, keywords, comment_tags, options):
    # Use the Django-Jinja2 -configured Jinja2 environment for extraction.
    # Otherwise mimicked from `jinja2.ext.babel_extract`
    environment = options['environment']

    source = fileobj.read().decode(settings.DEFAULT_CHARSET)
    node = environment.parse(source)
    tokens = list(environment.lex(environment.preprocess(source)))

    finder = _CommentFinder(tokens, comment_tags)
    for lineno, func, message in extract_from_ast(node, keywords):
        yield lineno, func, message, finder.find_comments(lineno)


def jinja2_extract(fileobj, keywords, comment_tags, options):
    """
    Extract messages from Jinja2 files, attempting to use a Django-Jinja2
    :param fileobj:
    :param keywords:
    :param comment_tags:
    :param options:
    :return:
    """
    try:
        from django_jinja.backend import Jinja2
        options = options.copy()
        options['environment'] = Jinja2.get_default().env
        extractor = django_jinja2_extract
    except (ImportError, ImproperlyConfigured) as exc:  # pragma: no cover
        warn('falling back to default jinja2 extractor due to %s' % exc)
        extractor = babel_extract
    return extractor(fileobj, keywords, comment_tags, options)



from babel.messages import Catalog
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils import translation

from i18nkit.utils import raise_if_no_module

try:
    import openpyxl
except ImportError:
    openpyxl = None


def write_catalog_workbook(outfp, catalog, languages=None, source_locale=None, prefill=True):
    raise_if_no_module('openpyxl')
    if not languages:
        languages = settings.LANGUAGES
    if not source_locale:
        source_locale = languages[0][0]

    workbook = openpyxl.Workbook()
    sheet = workbook.active
    target_locales = sorted([lc for (lc, name) in languages if lc != source_locale])
    sheet.append([source_locale] + target_locales)
    for message in catalog:
        original = message.id
        if not original:
            continue
        row = [original]
        if prefill:
            for locale in target_locales:
                with translation.override(locale):
                    translated = translation.gettext(original)
                    row.append((translated if (translated != original) else ''))
        sheet.append(row)
    workbook.save(outfp)


def read_catalog_workbook(infp, source_locale=None):
    raise_if_no_module('openpyxl')
    workbook = openpyxl.load_workbook(infp, read_only=True)
    catalogs = {}
    for sheet in workbook:
        languages = []
        for y, row in enumerate(sheet):
            texts = [cell.value for cell in row]
            if y == 0:
                languages = texts[:]
                if not source_locale:
                    source_locale = languages[0]
                for language in set(languages) - set(catalogs):
                    catalogs[language] = Catalog(language)
            else:
                texts = dict(zip(languages, texts))
                for language, text in texts.items():
                    if not text:
                        continue
                    strip_text = text.strip()
                    if not strip_text:
                        continue
                    catalogs[language].add(texts[source_locale], text)
    return catalogs



from django.core.management import BaseCommand

from i18nkit.excel import write_catalog_workbook
from i18nkit.utils import merge_catalog_files


class Command(BaseCommand):
    def add_arguments(self, parser):
        """
        :type parser: argparse.ArgumentParser
        """
        parser.add_argument('-i', '--input', nargs='*')
        parser.add_argument('-o', '--output')

    def handle(self, input, output, **options):
        merged_catalog = merge_catalog_files(input)
        if output.endswith('xlsx'):
            with open(output, 'wb') as outfp:
                write_catalog_workbook(outfp, merged_catalog)
                self.stdout.write('Wrote: %s' % outfp.name)
        else:  # pragma: no cover
            raise ValueError('unknown output type: %s' % output)



import os
from importlib import import_module

from babel.messages import Catalog
from babel.messages.pofile import read_po
from django.apps import apps as app_registry
from django.core.exceptions import ImproperlyConfigured


def merge_catalogs(catalogs):
    """
    Merge the given Catalogs, ruthlessly replacing (not augmenting) content.

    :param catalogs: List of catalogs
    :type catalogs: Iterable[babel.messages.Catalog]
    :return: New catalog
    :rtype: babel.messages.Catalog
    """
    merged_catalog = None
    for catalog in catalogs:
        if catalog is None:
            continue
        if merged_catalog is None:
            merged_catalog = Catalog(
                locale=str(catalog.locale or 'en'),
                charset=catalog.charset
            )
        assert catalog.charset == merged_catalog.charset
        for msg in catalog:
            if not msg.id:
                continue
            merged_catalog.delete(msg.id)
            merged_catalog[msg.id] = msg
    return merged_catalog


def merge_catalog_files(catalog_filenames):
    catalogs = []
    for filename in catalog_filenames:
        with open(filename, 'rb') as infp:
            catalogs.append(read_po(infp))
    return merge_catalogs(catalogs)


def yes(*args, **kwargs):
    return True


def get_paths(apps=(), dirs=(), dir_children=(), all_apps=False, dirname_filter=yes):
    paths = set()
    if apps or all_apps:
        for app_config in app_registry.get_app_configs():
            if all_apps or app_config.name in apps:
                paths.add(os.path.realpath(app_config.path))

    paths.update(set(os.path.realpath(dir) for dir in dirs))

    for parent_dir in dir_children:
        parent_dir = os.path.realpath(parent_dir)
        for dir in os.listdir(parent_dir):
            if os.path.isdir(dir) and dirname_filter(dir):
                paths.add(dir)

    return paths


def add_paths_options(parser):
    parser.add_argument('--all-apps', action='store_true')
    parser.add_argument('-a', '--app', action='append', dest='apps', default=[])
    parser.add_argument('-d', '--dir', action='append', dest='dirs', default=[])
    parser.add_argument(
        '--dir-children',
        action='append',
        dest='dir_children',
        help='add immediate directory children of this directory as dirs',
        default=[]
    )


def raise_if_no_module(module):
    try:
        import_module(module)
    except ImportError:
        raise ImproperlyConfigured("`%s` is required for this functionality" % module)


class DirectoryFilter(object):
    def __init__(self, ignore_dirs=(), default_ignore_dirs=True):
        self.ignore_dirs = ignore_dirs
        self.default_ignore_dirs = default_ignore_dirs

    def matches(self, path):
        basename = os.path.basename(path)
        if basename[0] in {'.', '_'}:
            return False
        if self.default_ignore_dirs:
            if basename in {'bower_components', 'node_modules', 'tests', 'htmlcov'}:
                return False
        return (basename not in self.ignore_dirs)

    def __call__(self, path):
        return self.matches(path)

    @classmethod
    def add_options(cls, parser):
        parser.add_argument('-I', '--ignore-dir', action='append', dest='ignore_dirs', default=[])
        parser.add_argument('--no-default-ignore-dirs', dest='default_ignore_dirs', action='store_false', default=True)



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='django-i18nkit',
    version='0.0.3',
    author='Aarni Koskela',
    author_email='akx@iki.fi',
    description='Internationalization utilities for Django.',
    license='MIT',
    packages=find_packages(exclude=["test*"]),
    zip_safe=False,
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    install_requires=[
        'babel<3.0',
        'Django>=1.8,<1.10',
        'django_babel>=0.5.0',
    ]
)
